==> fetch.php?media=uls%3Auls_update-1.9.5.tgz: update/von_1.9.4/sql/modetwert.sql <==
set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS modetwert//
create function modetwert(wid integer, anz integer, mode varchar(12)) returns varchar(20000)
  deterministic
  sql security invoker
begin
  declare wl, f, counter, z integer;
  declare w, wmin, wmax, wlast varbinary(20000);
  declare wsum2, wsum double;
  declare lastdate, d datetime(6);
  declare t enum('str','num','dat');
  declare selcur cursor for select wert, datum, length(0+replace(wert,'.','')) from twerte
    where werteindizesID = wid order by datum desc;
  declare selcurs cursor for select wert from twerte
    where werteindizesID = wid order by datum desc;
  declare continue handler for not found set z = 1;

  set counter = 0;
  set z = 0;
  select type from werteindizes, teststepdetails, einheiten
      where werteindizes.ID=wid and teststepdetailsID=teststepdetails.ID
        and einheitID=einheiten.ID
      into t;
  if t = 'num' then
    if mode = 'lastsize' then
      set anz = 1;
      set mode = 'firstsize';
    elseif mode = 'last' then
      set anz = 1;
      set mode = 'first';
    end if;
    open selcur;
    repeat
      fetch selcur into w, d, wl;
      if z != 1
       then
        if counter = 0 then
          set lastdate = d;
          set wmin = w;
          set wmax = w;
          set wsum = w;
          set wsum2 = w*w;
          set wlast = w;
          set f = wl;
        elseif counter = 1 && mode like 'last/avg%' then
          set wsum = w;
          set anz = anz + 1;
          if wl > f then
            set f = wl;
          end if;
        else
          set wsum = wsum + w;
          set wsum2 = wsum2 + w*w;
          if wl > f then
            set f = wl;
          end if;
          if 0+w < 0+wmin then
            set wmin = w;
          end if;
          if 0+w > 0+wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcur;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'avg' then
          set wsum = wsum / counter;
          return cast(round(wsum, f + floor(log10(counter)+2.3) - locate('.', wsum)) as char(30));
        when 'dev' then
          if counter > 1 then
            return cast(round(sqrt((wsum2 - wsum*wsum/counter)/(counter-1)), f + floor(log10(counter)+2.3) - locate('.', wsum/counter)) as char(30));
          else
            return '';
          end if;
        when 'last/avg' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1);
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'last/avg-1' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1) - 1;
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'sum' then
          return round(wsum, f + 1 - locate('.', wsum));
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'max-min' then
          set wsum = wmax - wmin;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first' then
          set wsum = wlast - w;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first+' then
          set wsum = wlast - w;
          if wsum >= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'last-first-' then
          set wsum = wlast - w;
          if wsum <= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        when 'grad' then
          return gradientzvb(wid, d, lastdate);
        when 'accel' then
          return accelz(wid, d, lastdate, '00:00:00', '24:00:00', '',
                        '', '', '', '', '', '', '', '', '', '', '');
        else
          return '';
      end case;
    else
      return '';
    end if;
  else
    if mode = 'size' OR mode = 'lastsize' then
      set anz = 1;
      set mode = 'firstsize';
    elseif mode = 'last' then
      set anz = 1;
      set mode = 'first';
    end if;
    open selcurs;
    repeat
      fetch selcurs into w;
      if z != 1
       then
        if counter = 0 then
          set wmin = w;
          set wmax = w;
          set wlast = w;
        else
          if w < wmin then
            set wmin = w;
          end if;
          if w > wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcurs;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        else
          return "";
      end case;
    else
      return "";
    end if;
  end if;
end//

GRANT EXECUTE on FUNCTION modetwert to logginglesen@'localhost', loggingtest@'localhost'//

==> fetch.php?media=uls%3Auls_update-1.9.6.tgz: update/von_1.9.5/sql/modetwert.sql <==
-- modetwert.sql
--
-- function modetwert(wid, anz, mode) returns varchar
-- wid: werteindizesID
-- anz: number of values
-- mode: aggregation mode
-- return: aggregated value of the last "anz" values with werteindizesID == "wid" (table twerte)

set @@character_set_client = 'utf8';

delimiter //

DROP FUNCTION IF EXISTS modetwert//
create function modetwert(wid integer, anz integer, mode varchar(12)) returns varchar(20000)
  deterministic
  sql security invoker
begin
  declare wl, f, counter, z integer;
  declare w, wmin, wmax, wlast varbinary(32000);
  declare wsum2, wsum double;
  declare lastdate, d datetime(6);
  declare t enum('str','num','dat');
  declare selcur cursor for select wert, datum, length(0+replace(wert,'.','')) from twerte
    where werteindizesID = wid order by datum desc;
  declare selcurs cursor for select wert from twerte
    where werteindizesID = wid order by datum desc;
  declare continue handler for not found set z = 1;

  set counter = 0;
  set z = 0;
  select type into t from werteindizes, teststepdetails, einheiten
      where werteindizes.ID=wid and teststepdetailsID=teststepdetails.ID
        and einheitID=einheiten.ID;
  if t = 'num' then
    if mode = 'lastsize' then
      set anz = 1;
      set mode = 'firstsize';
    elseif mode = 'last' then
      set anz = 1;
      set mode = 'first';
    end if;
    open selcur;
    repeat
      fetch selcur into w, d, wl;
      if z != 1
       then
        if counter = 0 then
          set lastdate = d;
          set wmin = w;
          set wmax = w;
          set wsum = w;
          set wsum2 = w*w;
          set wlast = w;
          set f = wl;
        elseif counter = 1 && mode like 'last/avg%' then
          set wsum = w;
          set anz = anz + 1;
          if wl > f then
            set f = wl;
          end if;
        else
          set wsum = wsum + w;
          set wsum2 = wsum2 + w*w;
          if wl > f then
            set f = wl;
          end if;
          if 0+w < 0+wmin then
            set wmin = w;
          end if;
          if 0+w > 0+wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcur;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'avg' then
          set wsum = wsum / counter;
          return cast(round(wsum, f + floor(log10(counter)+2.3) - locate('.', wsum)) as char(30));
        when 'dev' then
          if counter > 1 then
            return cast(round(sqrt((wsum2 - wsum*wsum/counter)/(counter-1)), f + floor(log10(counter)+2.3) - locate('.', wsum/counter)) as char(30));
          else
            return '';
          end if;
        when 'last/avg' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1);
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'last/avg-1' then
          if wsum != 0 AND counter > 1 then
            set wsum = wlast / wsum * (counter-1) - 1;
            return cast(round(wsum, f + 2 - locate('.', wsum)) as char(30));
          else
            return '';
          end if;
        when 'sum' then
          return round(wsum, f + 1 - locate('.', wsum));
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'max-min' then
          set wsum = wmax - wmin;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first' then
          set wsum = wlast - w;
          return round(wsum, f + 1 - locate('.', wsum));
        when 'last-first+' then
          set wsum = wlast - w;
          if wsum >= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'last-first-' then
          set wsum = wlast - w;
          if wsum <= 0 then
            return round(wsum, f + 1 - locate('.', wsum));
          else
            return '';
          end if;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        when 'grad' then
          return gradientzvb(wid, d, lastdate);
        when 'accel' then
          return accelz(wid, d, lastdate, '00:00:00', '24:00:00', '',
                        '', '', '', '', '', '', '', '', '', '', '');
        else
          return '';
      end case;
    else
      return '';
    end if;
  else
    if mode = 'size' OR mode = 'lastsize' then
      set anz = 1;
      set mode = 'firstsize';
    elseif mode = 'last' then
      set anz = 1;
      set mode = 'first';
    end if;
    open selcurs;
    repeat
      fetch selcurs into w;
      if z != 1
       then
        if counter = 0 then
          set wmin = w;
          set wmax = w;
          set wlast = w;
        else
          if w < wmin then
            set wmin = w;
          end if;
          if w > wmax then
            set wmax = w;
          end if;
        end if;
        set counter = counter + 1;
      end if;
      until (counter >= anz || z = 1)
    end repeat;
    close selcurs;
    if counter >= anz && counter > 0
     then
      case mode
        when 'count' then
          return counter;
        when 'min' then
          return wmin;
        when 'max' then
          return wmax;
        when 'first' then
          return w;
        when 'differ' then
          return w != wlast;
        when 'firstsize' then
          return length(w);
        else
          return "";
      end case;
    else
      return "";
    end if;
  end if;
end//

GRANT EXECUTE on FUNCTION modetwert to logginglesen@'localhost', loggingtest@'localhost'//
